
#include <iostream>
using namespace std;

#include "EventGenerators.h"
#include "Geometry.h"
using namespace Event;

#include <TRandom.h>


// -----------------------------------------------------------------------------
/// \brief Generate a position pos2 some fixed angular distance from pos1.
// -----------------------------------------------------------------------------
void
Event::GenerateEventNearPosition(
  const pointing &pos1, const double angdist, pointing &pos2)
{
  // Throw a point in a coordinate system where pos1 is the zenith.
  pos2.phi   = 360. * gRandom->Rndm() * degr2rad;
  pos2.theta = angdist;
  vec3 vloc  = pos2.to_vec3();

  // Rotate the position into the global coordinate system.
  Geo::Rotate(pos1, vloc);

  // Convert to a (theta, phi) pair.
  vec2pnt(vloc, pos2);
}


// -----------------------------------------------------------------------------
/// \brief Generate a position pos2 some fixed angular distance from pos1, with
///        a fixed "phase" [0,1] in the local azimuth.
// -----------------------------------------------------------------------------
void
Event::GenerateEventNearPosition(
  const pointing &pos1, const double angdist, const double phase,
  pointing &pos2)
{
  // Throw a point in a coordinate system where pos1 is the zenith.
  pos2.phi   = 360. * phase * degr2rad;
  pos2.theta = angdist;
  vec3 vloc  = pos2.to_vec3();

  // Rotate the position into the global coordinate system.
  Geo::Rotate(pos1, vloc);

  // Convert to a (theta, phi) pair.
  vec2pnt(vloc, pos2);
}


// -----------------------------------------------------------------------------
/// \brief Generate a position pos2 in a uniform disc around position pos1.
// -----------------------------------------------------------------------------
void
Event::GenerateEventWithinDisc(
  const pointing &pos1, const double angdist, pointing &pos2)
{
  // Throw a random direction inside a spherical cap of radius angdist.
  do {
    pos2.phi   = 360. * gRandom->Rndm() * degr2rad;
    pos2.theta = gRandom->Rndm() * angdist;
  }
  while (gRandom->Rndm() > sin(pos2.theta));

  // Rotate the position into the global coordinate system.
  vec3 vloc  = pos2.to_vec3();
  Geo::Rotate(pos1, vloc);

  // Convert to a (theta, phi) pair.
  vec2pnt(vloc, pos2);
}

