
#ifndef __Event_EventGenerators_h__
#define __Event_EventGenerators_h__

#include <healpix-cxx/healpix/healpix_map.h>
#include <healpix-cxx/cxxsupport/pointing.h>

#include <vector>

namespace Event {

  typedef std::vector< pointing > PointArray;
  typedef std::vector< pointing >::iterator PointArrayIter;
  typedef std::vector< pointing >::const_iterator PointArrayCIter;

  typedef std::vector< int > PixelArray;
  typedef std::vector< int >::iterator PixelArrayIter;
  typedef std::vector< int >::const_iterator PixelArrayCIter;

  template < class T > void
  GenerateEventFromSkymap(
    const Healpix_Map< T > &map, pointing &dir
    );

  template < class T > int
  GeneratePixelFromSkymap(
    const Healpix_Map< T > &map
    );

  template < class T > void
  GenerateEventListFromSkymap(
    const Healpix_Map< T > &map, const size_t N, PointArray &parray
    );

  template < class T > void
  GeneratePixelListFromSkymap(
    const Healpix_Map< T > &map, const size_t N, PixelArray &parray
    );

  void
  GenerateEventNearPosition(
    const pointing &pos1, const double angdist, pointing &pos2
    );

  void
  GenerateEventNearPosition(
    const pointing &pos1, const double angdist, const double phase, 
    pointing &pos2
    );

  void
  GenerateEventWithinDisc(
    const pointing &pos1, const double angdist, pointing &pos2
    );

  template < class T > int
  GeneratePixelNearPosition(
    const Healpix_Map< T > &map, const int pixel, const double angdist
    );

  template < class T > int
  GeneratePixelWithinDisc(
    const Healpix_Map< T > &map, const int pixel, const double angdist
    );

  template < class T> void
  PixelToPointArray(
    const Healpix_Map< T > &map, const PixelArray &pxarray, PointArray &ptarray
    );

  template < class T> void
  PointToPixelArray(
    const Healpix_Map< T > &map, const PointArray &ptarray, PixelArray &pxarray
    );

}

#include "EventGenerators.inl"

#endif // __Event_EventGenerators_h__

