
// C, C++, and STL includes.
#include <iostream>

// HEALPix C++ module includes.
#include <healpix-cxx/healpix/healpix_map.h>
#include <healpix-cxx/cxxsupport/pointing.h>
#include <healpix-cxx/cxxsupport/vec3.h>

// Local includes.
#include "skymap_functions.h"

using namespace std;

// -----------------------------------------------------------------------------
/// \brief Find the angular distance between two vectors on the unit sphere.
// -----------------------------------------------------------------------------
double
angdist(const vec3 &v1, const vec3 &v2)
{
   vec3 r1 = v1;   r1.Normalize();
   vec3 r2 = v2;   r2.Normalize();
   double dist = 0.0;
   double sprod = dotprod(r1, r2);

   if (sprod > 0.999) {             // Vectors almost parallel.
      vec3 vdiff  = r1 - r2;
      double diff = std::sqrt(dotprod(vdiff, vdiff));
      dist = 2.0 * std::asin(0.5 * diff);
   }
   else if (sprod < -0.999) {       // Vectors almost antiparallel.
      vec3 vsum  = r1 + r2;
      double sum = std::sqrt(dotprod(vsum, vsum));
      dist = pi - 2.0 * std::asin(0.5 * sum);
   }
   else {
      dist = std::acos(sprod);
   }

   return dist;
}

// -----------------------------------------------------------------------------
/// \brief Find the angular distance between two angles on the unit sphere.
// -----------------------------------------------------------------------------
double
angdist(const pointing &p1, const pointing &p2)
{
   return angdist(p1.to_vec3(), p2.to_vec3());
}

// -----------------------------------------------------------------------------
/// \brief Find the angular distance between two points on the unit sphere.
// -----------------------------------------------------------------------------
double
angdist(double th1, double ph1, double th2, double ph2)
{
   pointing p1(th1, ph1);
   pointing p2(th2, ph2);

   return angdist(p1, p2);
}

